"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompressionResources = void 0;
const zlib_1 = __importDefault(require("zlib"));
/**
 * Compress text using GZIP
 * @param text The text to compress
 * @returns Compressed text (base64)
 */
function compress(text) {
    return __awaiter(this, void 0, void 0, function* () {
        const buff = Buffer.from(text, "utf8");
        const compressed = yield new Promise((resolve, reject) => {
            zlib_1.default.gzip(buff, (err, res) => {
                if (err)
                    return reject(err);
                resolve(res);
            });
        });
        return compressed.toString("base64");
    });
}
/**
 * Decompress a compressed string (GZIP, base64)
 * @param text The compressed text
 * @returns Decompressed text
 */
function decompress(text) {
    return __awaiter(this, void 0, void 0, function* () {
        const compressedData = Buffer.from(text, "base64");
        const decompressedData = yield new Promise((resolve, reject) => {
            zlib_1.default.gunzip(compressedData, (err, res) => {
                if (err)
                    return reject(err);
                resolve(res);
            });
        });
        return decompressedData.toString("utf8");
    });
}
function getCompressionResources() {
    return {
        "compression/v2/compressText": compress,
        "compression/v2/decompressText": decompress
    };
}
exports.getCompressionResources = getCompressionResources;
