"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCryptoResources = void 0;
const iocane_1 = require("iocane");
const crypto_random_string_1 = __importDefault(require("crypto-random-string"));
const constants_1 = require("../core/constants");
let __derivationRoundsOverride = constants_1.CRYPTO_PBKDF2_ROUNDS;
function decryptData(data, password) {
    return (0, iocane_1.createAdapter)().decrypt(data, password);
}
function encryptData(data, password) {
    const adapter = (0, iocane_1.createAdapter)();
    if (__derivationRoundsOverride > 0) {
        adapter.setDerivationRounds(__derivationRoundsOverride);
    }
    return adapter.encrypt(data, password);
}
function getCryptoResources() {
    return {
        "crypto/v2/decryptBuffer": decryptData,
        "crypto/v2/encryptBuffer": encryptData,
        "crypto/v1/decryptText": decryptData,
        "crypto/v1/encryptText": encryptData,
        "crypto/v1/randomString": randomString,
        "crypto/v1/setDerivationRounds": setDerivationRounds
    };
}
exports.getCryptoResources = getCryptoResources;
function randomString(length) {
    return __awaiter(this, void 0, void 0, function* () {
        return (0, crypto_random_string_1.default)({
            length,
            characters: constants_1.CRYPTO_RANDOM_STRING_CHARS
        });
    });
}
function setDerivationRounds(rounds = null) {
    __derivationRoundsOverride = !rounds ? constants_1.CRYPTO_PBKDF2_ROUNDS : rounds;
}
