"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompressionResources = void 0;
const pako_1 = require("pako");
const encoding_1 = require("../../tools/encoding");
/**
 * Compress text using GZIP
 * @param text The text to compress
 * @returns Compressed text
 */
function compress(text) {
    return __awaiter(this, void 0, void 0, function* () {
        return Promise.resolve().then(() => {
            const output = (0, pako_1.gzip)(text, {
                level: 9
            });
            return (0, encoding_1.bytesToBase64)(output);
        });
    });
}
/**
 * Decompress a compressed string (GZIP)
 * @param text The compressed text
 * @returns Decompressed text
 */
function decompress(text) {
    return __awaiter(this, void 0, void 0, function* () {
        return Promise.resolve().then(() => {
            return (0, pako_1.ungzip)((0, encoding_1.base64ToBytes)(text), { to: "string" });
        });
    });
}
function getCompressionResources() {
    return {
        "compression/v2/compressText": compress,
        "compression/v2/decompressText": decompress
    };
}
exports.getCompressionResources = getCompressionResources;
