"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCryptoResources = void 0;
const web_1 = require("iocane/web");
const constants_1 = require("../core/constants");
const UINT16_MAX = 65535;
let __derivationRoundsOverride = constants_1.CRYPTO_PBKDF2_ROUNDS;
function decryptData(data, password) {
    return (0, web_1.createAdapter)().decrypt(data, password);
}
function encryptData(data, password) {
    const adapter = (0, web_1.createAdapter)();
    if (__derivationRoundsOverride > 0) {
        adapter.setDerivationRounds(__derivationRoundsOverride);
    }
    return adapter.encrypt(data, password);
}
function getCryptoResources() {
    return {
        "crypto/v2/decryptBuffer": decryptData,
        "crypto/v2/encryptBuffer": encryptData,
        "crypto/v1/decryptText": decryptData,
        "crypto/v1/encryptText": encryptData,
        "crypto/v1/randomString": randomString,
        "crypto/v1/setDerivationRounds": setDerivationRounds
    };
}
exports.getCryptoResources = getCryptoResources;
function randomString(length) {
    return Promise.resolve().then(() => {
        const randCharsLen = constants_1.CRYPTO_RANDOM_STRING_CHARS.length;
        const randArr = new Uint16Array(length);
        return randArr.reduce((output, nextVal) => {
            const ind = Math.floor((nextVal / UINT16_MAX) * randCharsLen);
            return `${output}${constants_1.CRYPTO_RANDOM_STRING_CHARS[ind]}`;
        }, "");
    });
}
function setDerivationRounds(rounds = null) {
    __derivationRoundsOverride = !rounds ? constants_1.CRYPTO_PBKDF2_ROUNDS : rounds;
}
