"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tools_1 = require("./tools");
const types_1 = require("../types");
const DIGIT = "\\d";
function createBaseFields(entry) {
    return [(0, tools_1.createFieldDescriptor)(entry, "Title", types_1.EntryPropertyType.Property, "title")];
}
function createCreditCardFields(entry) {
    return [
        ...createBaseFields(entry),
        (0, tools_1.createFieldDescriptor)(entry, "Card Holder", types_1.EntryPropertyType.Property, "username"),
        (0, tools_1.createFieldDescriptor)(entry, "Card Number", types_1.EntryPropertyType.Property, "password", {
            formatting: {
                format: [
                    { char: DIGIT, repeat: 4 },
                    { exactly: "-" },
                    { char: DIGIT, repeat: 4 },
                    { exactly: "-" },
                    { char: DIGIT, repeat: 4 },
                    { exactly: "-" },
                    { char: DIGIT, repeat: 4 }
                ],
                placeholder: "DDDD-DDDD-DDDD-DDDD"
            }
        }),
        (0, tools_1.createFieldDescriptor)(entry, "Card Type", types_1.EntryPropertyType.Property, "type", {
            formatting: {
                options: {
                    visa: "Visa",
                    mastercard: "Mastercard",
                    amex: "American Express",
                    discover: "Discover",
                    generic: "Generic"
                },
                defaultOption: "generic"
            }
        }),
        (0, tools_1.createFieldDescriptor)(entry, "CVV", types_1.EntryPropertyType.Property, "cvv", {
            formatting: {
                format: [{ char: DIGIT, repeat: 4 }],
                placeholder: "DDD or DDDD"
            },
            valueType: types_1.EntryPropertyValueType.Password
        }),
        (0, tools_1.createFieldDescriptor)(entry, "Valid From", types_1.EntryPropertyType.Property, "valid_from", {
            formatting: {
                format: [
                    { char: "[01]", repeat: 1 },
                    { char: DIGIT, repeat: 1 },
                    { exactly: "/" },
                    { char: "2", repeat: 1 },
                    { char: DIGIT, repeat: 3 }
                ],
                placeholder: "MM/YYYY"
            }
        }),
        (0, tools_1.createFieldDescriptor)(entry, "Expiry", types_1.EntryPropertyType.Property, "expiry", {
            formatting: {
                format: [
                    { char: "[01]", repeat: 1 },
                    { char: DIGIT, repeat: 1 },
                    { exactly: "/" },
                    { char: "2", repeat: 1 },
                    { char: DIGIT, repeat: 3 }
                ],
                placeholder: "MM/YYYY"
            }
        })
    ];
}
function createLoginFields(entry) {
    return [
        ...createBaseFields(entry),
        (0, tools_1.createFieldDescriptor)(entry, "Username", types_1.EntryPropertyType.Property, "username"),
        (0, tools_1.createFieldDescriptor)(entry, "Password", types_1.EntryPropertyType.Property, "password", {
            valueType: types_1.EntryPropertyValueType.Password
        })
    ];
}
function createNoteFields(entry) {
    return [
        ...createBaseFields(entry),
        (0, tools_1.createFieldDescriptor)(entry, "Note", types_1.EntryPropertyType.Property, "note", {
            valueType: types_1.EntryPropertyValueType.Note
        })
    ];
}
function createSSHKeyFields(entry) {
    return [
        ...createBaseFields(entry),
        (0, tools_1.createFieldDescriptor)(entry, "Public key", types_1.EntryPropertyType.Property, "publicKey"),
        (0, tools_1.createFieldDescriptor)(entry, "Private key", types_1.EntryPropertyType.Property, "privateKey", {
            valueType: types_1.EntryPropertyValueType.Note
        })
    ];
}
function createWebsiteFields(entry) {
    return [...createLoginFields(entry), (0, tools_1.createFieldDescriptor)(entry, "URL", types_1.EntryPropertyType.Property, "url")];
}
exports.default = {
    [types_1.EntryType.CreditCard]: createCreditCardFields,
    [types_1.EntryType.Login]: createLoginFields,
    [types_1.EntryType.Note]: createNoteFields,
    [types_1.EntryType.SSHKey]: createSSHKeyFields,
    [types_1.EntryType.Website]: createWebsiteFields
};
