"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const VaultFormat_1 = __importDefault(require("./VaultFormat"));
const Vault_1 = __importDefault(require("../core/Vault"));
const uuid_1 = require("../tools/uuid");
const appEnv_1 = require("../env/appEnv");
const channel_1 = require("../credentials/channel");
const common_1 = require("./common");
const signing_1 = require("./formatB/signing");
const compare_1 = require("./formatB/compare");
const merge_1 = require("./formatB/merge");
const conversion_1 = require("./formatB/conversion");
const history_1 = require("./formatB/history");
const date_1 = require("../tools/date");
const types_1 = require("../types");
function emptyVault() {
    return {
        id: null,
        a: {},
        g: [],
        e: [],
        c: (0, date_1.getDateString)()
    };
}
class VaultFormatB extends VaultFormat_1.default {
    static encodeRaw(rawContent, credentials) {
        const compress = (0, appEnv_1.getSharedAppEnv)().getProperty("compression/v2/compressText");
        const encrypt = (0, appEnv_1.getSharedAppEnv)().getProperty("crypto/v1/encryptText");
        const { masterPassword } = (0, channel_1.getCredentials)(credentials.id);
        return Promise.resolve()
            .then(() => (0, common_1.historyArrayToString)(rawContent))
            .then(history => compress(history))
            .then(compressed => encrypt(compressed, masterPassword))
            .then(signing_1.sign);
    }
    static extractSharesFromHistory(history) {
        return {};
    }
    static getFormatID() {
        return types_1.VaultFormatID.B;
    }
    static historiesDiffer(historyA, historyB) {
        return (0, compare_1.historiesDiffer)(historyA, historyB);
    }
    static isEncrypted(contents) {
        return (0, signing_1.vaultContentsEncrypted)(contents);
    }
    static parseEncrypted(encryptedContent, credentials) {
        const decompress = (0, appEnv_1.getSharedAppEnv)().getProperty("compression/v2/decompressText");
        const decrypt = (0, appEnv_1.getSharedAppEnv)().getProperty("crypto/v1/decryptText");
        const { masterPassword } = (0, channel_1.getCredentials)(credentials.id);
        return Promise.resolve()
            .then(() => {
            if (!(0, signing_1.hasValidSignature)(encryptedContent)) {
                throw new Error("No valid signature in vault");
            }
            return (0, signing_1.stripSignature)(encryptedContent);
        })
            .then(encryptedData => decrypt(encryptedData, masterPassword))
            .then((decrypted) => __awaiter(this, void 0, void 0, function* () {
            const decompressed = yield decompress(decrypted);
            return (0, common_1.historyStringToArray)(decompressed, types_1.VaultFormatID.B);
        }));
    }
    static prepareHistoryForMerge(history) {
        return history;
    }
    static vaultFromMergedHistories(base, incoming) {
        const baseRaw = JSON.parse(base[0]);
        const incomingRaw = JSON.parse(incoming[0]);
        const merged = (0, merge_1.mergeRawVaults)(baseRaw, incomingRaw);
        const vault = new Vault_1.default();
        vault._format = new VaultFormatB(merged);
        return vault;
    }
    constructor(source = emptyVault()) {
        super(source);
    }
    cloneEntry(entry, targetGroupID) { }
    cloneGroup(group, targetGroupID) {
        const newGroup = JSON.parse(JSON.stringify(group));
        newGroup.id = (0, uuid_1.generateUUID)();
        newGroup.g = targetGroupID;
        this.source.g.push(newGroup);
        // clone entries
        const childEntries = this.source.e
            .filter(entry => entry.g === group.id)
            .map(entry => {
            const newEntry = JSON.parse(JSON.stringify(entry));
            newEntry.g = newGroup.id;
            return newEntry;
        });
        this.source.e.push(...childEntries);
        // clone groups
        this.source.g.forEach(childGroup => {
            if (childGroup.g === group.id) {
                this.cloneGroup(childGroup, newGroup.id);
            }
        });
    }
    createEntry(groupID, entryID) {
        const entry = {
            id: entryID,
            g: groupID,
            p: {},
            a: {}
        };
        this.source.e.push(entry);
    }
    createGroup(parentID, groupID) {
        const group = {
            id: groupID,
            g: parentID,
            t: "",
            a: {}
        };
        this.source.g.push(group);
    }
    deleteEntry(entryID) {
        const ind = this.source.e.findIndex(entry => entry.id === entryID);
        if (ind >= 0) {
            this.source.e.splice(ind, 1);
        }
    }
    deleteEntryAttribute(entryID, attribute) {
        const entry = this.source.e.find((e) => e.id === entryID);
        if (!entry.a[attribute])
            return;
        entry.a[attribute].deleted = (0, date_1.getTimestamp)();
    }
    deleteEntryProperty(entryID, property) {
        const entry = this.source.e.find(e => e.id === entryID);
        if (!entry.p[property])
            return;
        entry.p[property].deleted = (0, date_1.getTimestamp)();
    }
    deleteGroup(groupID) {
        const ind = this.source.g.findIndex(group => group.id === groupID);
        if (ind >= 0) {
            this.source.g.splice(ind, 1);
        }
    }
    deleteGroupAttribute(groupID, attribute) {
        const group = this.source.g.find(g => g.id === groupID);
        if (!group.a[attribute])
            return;
        group.a[attribute].deleted = (0, date_1.getTimestamp)();
    }
    deleteVaultAttribute(attribute) {
        if (!this.source.a[attribute])
            return;
        this.source.a[attribute].deleted = (0, date_1.getTimestamp)();
    }
    erase() {
        super.erase();
        Object.assign(this.source, emptyVault());
        this.emit("erased");
    }
    execute(commandOrCommands) {
        let command;
        if (Array.isArray(commandOrCommands)) {
            if (commandOrCommands.length !== 1) {
                throw new Error(`Format-B commands array must contain a single command, received: ${commandOrCommands.length}`);
            }
            command = commandOrCommands[0];
        }
        else {
            command = commandOrCommands;
        }
        this.source = JSON.parse(command);
        this.dirty = true;
        this.emit("commandsExecuted");
    }
    findEntryByID(id) {
        return this.source.e.find(entry => entry.id === id) || null;
    }
    findGroupByID(id) {
        return this.source.g.find(group => group.id === id) || null;
    }
    findGroupContainingEntryID(id) {
        const matchingEntry = this.getAllEntries().find(entry => entry.id === id);
        if (matchingEntry) {
            return this.getAllGroups().find(group => group.id === matchingEntry.g) || null;
        }
        return null;
    }
    findGroupContainingGroupID(id) {
        const groups = this.getAllGroups();
        const matchingGroup = groups.find(group => group.id === id);
        if (!matchingGroup)
            return null;
        return groups.find(group => group.id === matchingGroup.g) || null;
    }
    generateID() {
        this.source.id = (0, uuid_1.generateUUID)();
    }
    getAllEntries(parentID = null) {
        const source = this.source;
        return parentID === null ? source.e : source.e.filter(entry => entry.g === parentID);
    }
    getAllGroups(parentID = null) {
        const source = this.source;
        return parentID === null ? source.g : source.g.filter(group => group.g === parentID);
    }
    getEntryAttributes(entrySource) {
        return (0, conversion_1.valuesObjectToKeyValueObject)(entrySource.a);
    }
    getEntryChanges(entrySource) {
        return Object.keys(entrySource.p).reduce((changes, property) => [
            ...changes,
            ...entrySource.p[property].history.map((histItem) => {
                const change = {
                    property,
                    type: histItem.updated === entrySource.p[property].created
                        ? types_1.EntryChangeType.Created
                        : types_1.EntryChangeType.Modified,
                    ts: histItem.updated,
                    value: histItem.value
                };
                return change;
            }),
            ...(entrySource.p[property].deleted
                ? [
                    {
                        property,
                        type: types_1.EntryChangeType.Deleted,
                        ts: entrySource.p[property].deleted,
                        value: null
                    }
                ]
                : [])
        ], []);
    }
    getEntryProperties(entrySource) {
        return (0, conversion_1.valuesObjectToKeyValueObject)(entrySource.p);
    }
    getFormat() {
        return VaultFormatB;
    }
    getGroupAttributes(groupSource) {
        return (0, conversion_1.valuesObjectToKeyValueObject)(groupSource.a);
    }
    getGroupTitle(groupSource) {
        return groupSource.t;
    }
    getHistory() {
        const hist = [JSON.stringify(this.source)];
        hist.format = types_1.VaultFormatID.B;
        return hist;
    }
    getItemID(itemSource) {
        return itemSource.id;
    }
    getItemParentID(itemSource) {
        return itemSource.g;
    }
    getVaultAttributes() {
        return (0, conversion_1.valuesObjectToKeyValueObject)(this.source.a);
    }
    getVaultID() {
        return this.source.id;
    }
    initialise() {
        Object.assign(this.source, {
            a: this.source.a || {},
            g: this.source.g || [],
            e: this.source.e || []
        });
        this.generateID();
    }
    moveEntry(entryID, groupID) {
        const entry = this.source.e.find((e) => e.id === entryID);
        entry.g = groupID;
    }
    moveGroup(groupID, newParentID) {
        const group = this.source.g.find((g) => g.id === groupID);
        group.g = newParentID;
    }
    optimise() { }
    setEntryAttribute(entryID, attribute, value) {
        const entry = this.source.e.find((e) => e.id === entryID);
        if (!entry.a[attribute]) {
            entry.a[attribute] = (0, history_1.newRawValue)(value);
        }
        else {
            const item = entry.a[attribute];
            item.history.unshift((0, history_1.valueToHistoryItem)(item));
            item.value = value;
            item.updated = (0, date_1.getTimestamp)();
        }
    }
    setEntryProperty(entryID, property, value) {
        const entry = this.source.e.find((e) => e.id === entryID);
        if (!entry.p[property]) {
            entry.p[property] = (0, history_1.newRawValue)(value);
        }
        else {
            const item = entry.p[property];
            item.history.unshift((0, history_1.valueToHistoryItem)(item));
            item.value = value;
            item.updated = (0, date_1.getTimestamp)();
        }
    }
    setGroupAttribute(groupID, attribute, value) {
        const group = this.source.g.find((g) => g.id === groupID);
        if (!group.a[attribute]) {
            group.a[attribute] = (0, history_1.newRawValue)(value);
        }
        else {
            const item = group.a[attribute];
            item.history.unshift((0, history_1.valueToHistoryItem)(item));
            item.value = value;
            item.updated = (0, date_1.getTimestamp)();
        }
    }
    setGroupTitle(groupID, title) {
        const group = this.source.g.find((g) => g.id === groupID);
        group.t = title;
    }
    setVaultAttribute(key, value) {
        if (!this.source.a[key]) {
            this.source.a[key] = (0, history_1.newRawValue)(value);
        }
        else {
            const item = this.source.a[key];
            item.history.unshift((0, history_1.valueToHistoryItem)(item));
            item.value = value;
            item.updated = (0, date_1.getTimestamp)();
        }
    }
}
exports.default = VaultFormatB;
