"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeTitleGroup = exports.executeSetGroupAttribute = exports.executeSetEntryProperty = exports.executeSetEntryAttribute = exports.executeSetArchiveAttribute = exports.executePad = exports.executeMoveGroup = exports.executeMoveEntry = exports.executeFormat = exports.executeDeleteGroupAttribute = exports.executeDeleteGroup = exports.executeDeleteEntryProperty = exports.executeDeleteEntryAttribute = exports.executeDeleteEntry = exports.executeDeleteArchiveAttribute = exports.executeCreateGroup = exports.executeCreateEntry = exports.executeComment = exports.executeArchiveID = void 0;
const tools_1 = require("./tools");
const types_1 = require("../../types");
function executeArchiveID(vault, opts, id) {
    if (opts.shareID) {
        // Don't set vault ID from a share
        return;
    }
    if (vault.id) {
        // ID already set
        throw new Error("Vault ID already set");
    }
    vault.id = id;
}
exports.executeArchiveID = executeArchiveID;
function executeComment() {
    // Comment has no action
}
exports.executeComment = executeComment;
function executeCreateEntry(archive, opts, groupID, entryID) {
    archive.groups = archive.groups || [];
    const entry = {
        id: entryID,
        properties: {
            title: ""
        },
        shareID: opts.shareID,
        parentID: groupID
    };
    if (opts.permissions) {
        entry.permissions = opts.permissions;
    }
    const group = (0, tools_1.findGroupByID)(archive.groups, groupID);
    if (!group) {
        throw new Error(`Invalid group ID: ${groupID}`);
    }
    group.entries = group.entries || [];
    group.entries.push(entry);
}
exports.executeCreateEntry = executeCreateEntry;
function executeCreateGroup(archive, opts, parentID, newID) {
    archive.groups = archive.groups || [];
    const group = {
        id: newID,
        title: "New group",
        shareID: opts.shareID,
        parentID
    };
    if (opts.permissions) {
        group.permissions = opts.permissions;
    }
    if (parentID === "0") {
        archive.groups.push(group);
    }
    else {
        const parentGroup = (0, tools_1.findGroupByID)(archive.groups, parentID);
        if (!parentGroup) {
            throw new Error(`Invalid parent group ID: ${parentID}`);
        }
        parentGroup.groups = parentGroup.groups || [];
        parentGroup.groups.push(group);
    }
}
exports.executeCreateGroup = executeCreateGroup;
function executeDeleteArchiveAttribute(archive, opts, attribute) {
    const attributes = archive.attributes || {};
    if (attributes.hasOwnProperty(attribute) !== true) {
        throw new Error(`Vault contains no such attribute: ${attribute}`);
    }
    const deleted = delete attributes[attribute];
    if (!deleted) {
        throw new Error(`Failed deleting attribute: ${attribute}`);
    }
}
exports.executeDeleteArchiveAttribute = executeDeleteArchiveAttribute;
function executeDeleteEntry(archive, opts, entryID) {
    archive.groups = archive.groups || [];
    const { group, index } = (0, tools_1.findGroupContainingEntryID)(archive.groups, entryID);
    if (!group) {
        throw new Error(`Failed deleting entry: Invalid entry ID: ${entryID}`);
    }
    group.entries.splice(index, 1);
}
exports.executeDeleteEntry = executeDeleteEntry;
function executeDeleteEntryAttribute(archive, opts, entryID, attribute) {
    archive.groups = archive.groups || [];
    const entry = (0, tools_1.findEntryByID)(archive.groups, entryID);
    if (!entry) {
        throw new Error("Entry not found for ID");
    }
    entry.attributes = entry.attributes || {};
    const value = entry.attributes[attribute];
    const deleted = delete entry.attributes[attribute];
    if (!deleted) {
        throw new Error("Failed deleting attribute");
    }
    entry.history = entry.history || [];
    entry.history.push((0, tools_1.generateEntryLegacyHistoryItem)(attribute, types_1.EntryPropertyType.Attribute, value, null));
}
exports.executeDeleteEntryAttribute = executeDeleteEntryAttribute;
function executeDeleteEntryProperty(archive, opts, entryID, property) {
    archive.groups = archive.groups || [];
    const entry = (0, tools_1.findEntryByID)(archive.groups, entryID);
    if (!entry) {
        throw new Error(`Entry not found for ID: ${entryID}`);
    }
    entry.properties = entry.properties || {};
    const value = entry.properties[property];
    const deleted = delete entry.properties[property];
    if (!deleted) {
        throw new Error(`Failed deleting property: ${property}`);
    }
    entry.history = entry.history || [];
    entry.history.push((0, tools_1.generateEntryLegacyHistoryItem)(property, types_1.EntryPropertyType.Property, value, null));
}
exports.executeDeleteEntryProperty = executeDeleteEntryProperty;
function executeDeleteGroup(archive, opts, groupID) {
    archive.groups = archive.groups || [];
    const { group, index } = (0, tools_1.findGroupContainingGroupID)(archive, groupID);
    if (!group) {
        throw new Error(`Invalid group ID: ${groupID}`);
    }
    group.groups.splice(index, 1);
}
exports.executeDeleteGroup = executeDeleteGroup;
function executeDeleteGroupAttribute(archive, opts, groupID, attribute) {
    archive.groups = archive.groups || [];
    const group = (0, tools_1.findGroupByID)(archive.groups, groupID);
    if (!group) {
        throw new Error(`Group not found for ID: ${groupID}`);
    }
    group.attributes = group.attributes || {};
    const deleted = delete group.attributes[attribute];
    if (!deleted) {
        throw new Error(`Failed deleting attribute: ${attribute}`);
    }
}
exports.executeDeleteGroupAttribute = executeDeleteGroupAttribute;
function executeFormat(archive, opts, format) {
    if (opts.shareID) {
        // Don't set archive format from a share
        return;
    }
    if (archive.format) {
        throw new Error("Format already set");
    }
    archive.format = format;
}
exports.executeFormat = executeFormat;
function executeMoveEntry(archive, opts, entryID, groupID) {
    archive.groups = archive.groups || [];
    const { group: originGroup, index: originIndex } = (0, tools_1.findGroupContainingEntryID)(archive.groups, entryID);
    if (!originGroup) {
        throw new Error(`Invalid entry ID: ${entryID}`);
    }
    const targetGroup = (0, tools_1.findGroupByID)(archive.groups, groupID);
    if (!targetGroup) {
        throw new Error("Invalid group ID");
    }
    const [movedEntry] = originGroup.entries.splice(originIndex, 1);
    targetGroup.entries = targetGroup.entries || [];
    targetGroup.entries.push(movedEntry);
    movedEntry.parentID = groupID;
}
exports.executeMoveEntry = executeMoveEntry;
function executeMoveGroup(archive, opts, groupID, targetGroupID) {
    archive.groups = archive.groups || [];
    const { group: originGroup, index: originIndex } = (0, tools_1.findGroupContainingGroupID)(archive, groupID);
    if (!originGroup) {
        throw new Error(`Invalid group ID: ${groupID}`);
    }
    const targetGroup = targetGroupID.length === 1 && parseInt(targetGroupID, 10) === 0
        ? archive
        : (0, tools_1.findGroupByID)(archive.groups, targetGroupID);
    if (!targetGroup) {
        throw new Error(`Invalid target group ID: ${targetGroupID}`);
    }
    const [movedGroup] = originGroup.groups.splice(originIndex, 1);
    targetGroup.groups = targetGroup.groups || [];
    targetGroup.groups.push(movedGroup);
    movedGroup.parentID = targetGroupID;
}
exports.executeMoveGroup = executeMoveGroup;
function executePad() {
    // Comment has no action
}
exports.executePad = executePad;
function executeSetArchiveAttribute(archive, opts, attribute, value) {
    archive.attributes = archive.attributes || {};
    archive.attributes[attribute] = value;
}
exports.executeSetArchiveAttribute = executeSetArchiveAttribute;
function executeSetEntryAttribute(archive, opts, entryID, attribute, value) {
    archive.groups = archive.groups || [];
    const entry = (0, tools_1.findEntryByID)(archive.groups, entryID);
    if (!entry) {
        throw new Error(`Entry not found for ID: ${entryID}`);
    }
    entry.attributes = entry.attributes || {};
    const previousValue = typeof entry.attributes[attribute] === "string" ? entry.attributes[attribute] : null;
    entry.attributes[attribute] = value;
    entry.history = entry.history || [];
    entry.history.push((0, tools_1.generateEntryLegacyHistoryItem)(attribute, types_1.EntryPropertyType.Attribute, previousValue, value));
}
exports.executeSetEntryAttribute = executeSetEntryAttribute;
function executeSetEntryProperty(archive, opts, entryID, property, value) {
    archive.groups = archive.groups || [];
    const entry = (0, tools_1.findEntryByID)(archive.groups, entryID);
    if (!entry) {
        throw new Error(`Entry not found for ID: ${entryID}`);
    }
    entry.properties = entry.properties || {};
    const previousValue = typeof entry.properties[property] === "string" ? entry.properties[property] : null;
    entry.properties[property] = value;
    entry.history = entry.history || [];
    entry.history.push((0, tools_1.generateEntryLegacyHistoryItem)(property, types_1.EntryPropertyType.Property, previousValue, value));
}
exports.executeSetEntryProperty = executeSetEntryProperty;
function executeSetGroupAttribute(archive, opts, groupID, attribute, value) {
    archive.groups = archive.groups || [];
    const group = (0, tools_1.findGroupByID)(archive.groups, groupID);
    if (!group) {
        throw new Error(`Group not found for ID: ${groupID}`);
    }
    group.attributes = group.attributes || {};
    group.attributes[attribute] = value;
}
exports.executeSetGroupAttribute = executeSetGroupAttribute;
function executeTitleGroup(archive, opts, groupID, title) {
    archive.groups = archive.groups || [];
    const group = (0, tools_1.findGroupByID)(archive.groups, groupID);
    if (!group) {
        throw new Error(`Group not found for ID: ${groupID}`);
    }
    group.title = title;
}
exports.executeTitleGroup = executeTitleGroup;
