"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.valuesObjectToKeyValueObject = exports.isFormatBGroup = exports.isFormatBEntry = exports.convertFormatAVault = exports.convertFormatAGroup = exports.convertFormatAEntry = void 0;
const history_1 = require("./history");
const date_1 = require("../../tools/date");
const tools_1 = require("../formatA/tools");
const types_1 = require("../../types");
function convertFormatAEntry(entry) {
    const changes = entry.history || [];
    const formatBEntry = {
        id: entry.id,
        a: flatKeyValueObjectToValuesObject(entry.attributes || {}),
        p: flatKeyValueObjectToValuesObject(entry.properties || {}),
        g: entry.parentID
    };
    for (const key in formatBEntry.a) {
        const propChanges = changes.filter(change => change.property === key && change.propertyType === types_1.EntryPropertyType.Attribute);
        formatBEntry.a[key].history = propChanges.map(propChange => ({
            value: propChange.newValue,
            updated: null
        }));
    }
    for (const key in formatBEntry.p) {
        const propChanges = changes.filter(change => change.property === key && change.propertyType === types_1.EntryPropertyType.Property);
        formatBEntry.p[key].history = propChanges.map(propChange => ({
            value: propChange.newValue,
            updated: null
        }));
    }
    return formatBEntry;
}
exports.convertFormatAEntry = convertFormatAEntry;
function convertFormatAGroup(group) {
    return {
        id: group.id,
        a: flatKeyValueObjectToValuesObject(group.attributes || {}),
        t: group.title,
        g: group.parentID
    };
}
exports.convertFormatAGroup = convertFormatAGroup;
function convertFormatAVault(vault) {
    return {
        id: vault.id,
        a: flatKeyValueObjectToValuesObject(vault.attributes || {}),
        g: (0, tools_1.getAllGroups)(vault).map(group => convertFormatAGroup(group)),
        e: (0, tools_1.getAllEntries)(vault).map(entry => convertFormatAEntry(entry)),
        c: (0, date_1.getDateString)()
    };
}
exports.convertFormatAVault = convertFormatAVault;
function flatKeyValueObjectToValuesObject(obj) {
    return Object.keys(obj).reduce((output, key) => Object.assign(output, {
        [key]: (0, history_1.newRawValue)(obj[key])
    }), {});
}
function isFormatBEntry(entry) {
    return entry.g !== undefined;
}
exports.isFormatBEntry = isFormatBEntry;
function isFormatBGroup(group) {
    return group.g !== undefined;
}
exports.isFormatBGroup = isFormatBGroup;
/**
 * Get a key-value object from a Format-B values object.
 * This strips deleted properties.
 * @param valuesObj
 */
function valuesObjectToKeyValueObject(valuesObj) {
    return Object.keys(valuesObj).reduce((output, key) => {
        if (valuesObj[key].deleted)
            return output;
        return Object.assign(output, {
            [key]: valuesObj[key].value
        });
    }, {});
}
exports.valuesObjectToKeyValueObject = valuesObjectToKeyValueObject;
