"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.valueToHistoryItem = exports.trimValueHistory = exports.sortValueHistory = exports.newRawValue = exports.mergeValues = exports.cloneValue = exports.MAX_PROPERTY_HISTORY = exports.MAX_ATTRIBUTE_HISTORY = void 0;
const date_1 = require("../../tools/date");
const types_1 = require("../../types");
exports.MAX_ATTRIBUTE_HISTORY = 3;
exports.MAX_PROPERTY_HISTORY = 5;
function cloneValue(value) {
    return {
        value: value.value,
        created: value.created,
        updated: value.updated,
        history: value.history.map(item => Object.assign({}, item))
    };
}
exports.cloneValue = cloneValue;
function filterDuplicateHistoryItems(items) {
    return items.filter((thisItem, thisIndex, itemArr) => itemArr.findIndex(sub => sub.value === thisItem.value && sub.updated === thisItem.updated) === thisIndex);
}
function mergeValues(value1, value2, type) {
    const mostRecentValue = value1.updated > value2.updated || value1.updated === value2.updated ? value1 : value2;
    const olderValue = value1 === mostRecentValue ? value2 : value1;
    const newHistory = sortValueHistory(filterDuplicateHistoryItems([valueToHistoryItem(olderValue), ...value1.history, ...value2.history]));
    return {
        value: mostRecentValue.value,
        created: mostRecentValue.created,
        updated: mostRecentValue.updated,
        history: trimValueHistory(newHistory, type)
    };
}
exports.mergeValues = mergeValues;
function newRawValue(value) {
    const ts = (0, date_1.getTimestamp)();
    return {
        value,
        created: ts,
        updated: ts,
        history: []
    };
}
exports.newRawValue = newRawValue;
function sortValueHistory(history) {
    return history.sort((a, b) => {
        if (a.updated > b.updated) {
            return -1;
        }
        else if (b.updated > a.updated) {
            return 1;
        }
        return 0;
    });
}
exports.sortValueHistory = sortValueHistory;
function trimValueHistory(history, type) {
    const max = type === types_1.EntryPropertyType.Attribute ? exports.MAX_ATTRIBUTE_HISTORY : exports.MAX_PROPERTY_HISTORY;
    const sorted = sortValueHistory(history);
    return sorted.slice(0, max);
}
exports.trimValueHistory = trimValueHistory;
function valueToHistoryItem(value) {
    return {
        value: value.value,
        updated: value.updated
    };
}
exports.valueToHistoryItem = valueToHistoryItem;
