"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VaultEntrySearch = void 0;
const BaseSearch_1 = require("./BaseSearch");
const entry_1 = require("../tools/entry");
function extractEntries(vault, memory) {
    return __awaiter(this, void 0, void 0, function* () {
        // Get scores
        const scoresRaw = yield memory.getValue(`bcup_search_${vault.id}`);
        let vaultScore = {};
        if (scoresRaw) {
            try {
                const scores = JSON.parse(scoresRaw);
                vaultScore = scores;
            }
            catch (err) { }
        }
        // Get entries
        return vault
            .getAllEntries()
            .filter((entry) => entry.isInTrash() === false)
            .map((entry) => {
            const properties = entry.getProperties();
            const urls = (0, entry_1.getEntryURLs)(properties, entry_1.EntryURLType.General);
            return {
                id: entry.id,
                properties,
                entryType: entry.getType(),
                urls,
                groupID: entry.getGroup().id,
                vaultID: vault.id,
                domainScores: vaultScore[entry.id] || {}
            };
        });
    });
}
class VaultEntrySearch extends BaseSearch_1.BaseSearch {
    constructor(vaults, memory, searcherFactory) {
        super(vaults, extractEntries, memory, searcherFactory);
    }
}
exports.VaultEntrySearch = VaultEntrySearch;
