"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEncoded = exports.getUniqueID = exports.encodeStringValue = exports.encodeBase64String = exports.decodeStringValue = exports.decodeBase64String = exports.bytesToBase64 = exports.base64ToBytes = exports.ENCODED_STRING_PREFIX = void 0;
const uuid_1 = require("./uuid");
const appEnv_1 = require("../env/appEnv");
const ENCODED_STRING_PATTERN = /^utf8\+base64:(|[a-zA-Z0-9+\/=]+)$/;
exports.ENCODED_STRING_PREFIX = "utf8+base64:";
/**
 * Decode a base64 string to a typed array
 * @param b64 The base64 string
 * @returns A typed array
 */
function base64ToBytes(b64) {
    const decode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/base64ToBytes");
    return decode(b64);
}
exports.base64ToBytes = base64ToBytes;
/**
 * Encode a typed array to base64
 * @param uint8Arr The array of bytes to encode
 * @returns A base64 encoded string
 */
function bytesToBase64(uint8Arr) {
    const encode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/bytesToBase64");
    return encode(uint8Arr);
}
exports.bytesToBase64 = bytesToBase64;
/**
 * Decode a base64 string
 * @param b64 The base64 string
 * @returns The decoded string
 */
function decodeBase64String(b64) {
    const decode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/base64ToText");
    return decode(b64);
}
exports.decodeBase64String = decodeBase64String;
/**
 * Decode an encoded property value
 * @param value The encoded value
 * @returns The decoded value
 */
function decodeStringValue(value) {
    if (isEncoded(value) !== true) {
        throw new Error("Cannot decode: provided value is not encoded");
    }
    const newValue = value.substr(exports.ENCODED_STRING_PREFIX.length);
    const decode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/base64ToText");
    return decode(newValue);
}
exports.decodeStringValue = decodeStringValue;
/**
 * Encode a base64 string
 * @param text The raw text to encode
 * @returns A base64 encoded string
 */
function encodeBase64String(text) {
    const encode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/textToBase64");
    return encode(text);
}
exports.encodeBase64String = encodeBase64String;
/**
 * Encode a raw value into safe storage form
 * Uses base64 for encoding
 * @param value The raw value to encode
 * @returns The encoded result
 */
function encodeStringValue(value) {
    const encode = (0, appEnv_1.getSharedAppEnv)().getProperty("encoding/v1/textToBase64");
    return `${exports.ENCODED_STRING_PREFIX}${encode(value)}`;
}
exports.encodeStringValue = encodeStringValue;
/**
 * Get a unique identifier (UUID v4)
 * @returns A unique identifier
 */
function getUniqueID() {
    return (0, uuid_1.generateUUID)();
}
exports.getUniqueID = getUniqueID;
/**
 * Check if a string value is encoded
 * @param text The value to check
 * @returns True if the text is encoded
 */
function isEncoded(text) {
    return ENCODED_STRING_PATTERN.test(text);
}
exports.isEncoded = isEncoded;
