"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeSharePrefix = exports.prependSharePrefix = exports.moveGroupBetweenVaults = exports.mergeHistories = void 0;
const Vault_1 = __importDefault(require("../core/Vault"));
const SHARE_COMMAND_EXP = /^\$[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}\s/;
function mergeHistories(initial, incoming) {
    // @todo Share merging
    return [];
}
exports.mergeHistories = mergeHistories;
/**
 * Move a group between archives
 * @param movingGroup The group to move
 * @param targetGroup The group to move to
 * @throws {Error} Throws if the remote type is not recognised
 */
function moveGroupBetweenVaults(movingGroup, targetGroup) {
    // Clone
    const targetVault = targetGroup instanceof Vault_1.default ? targetGroup : targetGroup.vault;
    targetVault.format.cloneGroup(movingGroup._source, targetGroup instanceof Vault_1.default ? "0" : targetGroup.id);
    // Delete original
    movingGroup.delete(/* skip trash */ true);
}
exports.moveGroupBetweenVaults = moveGroupBetweenVaults;
/**
 * Prepend the share prefix to every line that doesn't have it
 * @param history Array of history lines
 * @returns Prefixed history lines
 */
function prependSharePrefix(history, shareID) {
    return history.map(line => (SHARE_COMMAND_EXP.test(line) ? line : `$${shareID} ${line}`));
}
exports.prependSharePrefix = prependSharePrefix;
/**
 * Remove the share prefix to every line that has it
 * @param history Array of history lines
 * @returns Non-prefixed history lines
 */
function removeSharePrefix(history) {
    return history.map(line => (SHARE_COMMAND_EXP.test(line) ? line.replace(SHARE_COMMAND_EXP, "") : line));
}
exports.removeSharePrefix = removeSharePrefix;
