import Vault from "./Vault";
/**
 * Base vault member class (for Entry, Group etc.)
 */
export default class VaultItem {
    _boundUpdateRefs: () => void;
    _source: any;
    /**
     * Reference to the containing vault
     * @protected
     * @memberof VaultItem
     */
    _vault: Vault;
    /**
     * Constructor for the vault member base class
     * @param {Vault} vault Vault reference
     * @param {Object} source Remote source object reference
     */
    constructor(vault: Vault, source: any);
    /**
     * The ID of the entry or group
     * @readonly
     * @memberof VaultItem
     */
    get id(): string;
    /**
     * The current granted permissions
     * @memberof VaultItem
     */
    get permissions(): Array<string>;
    /**
     * The vault this item belongs to
     * @readonly
     * @memberof VaultItem
     */
    get vault(): Vault;
    /**
     * Grant a new permission to the member
     * @param perm The permission to grant
     * @memberof VaultItem
     */
    grantPermission(perm: string): void;
    /**
     * Check if the member has a permission
     * @param perm The permission to check for
     * @memberof VaultItem
     */
    hasPermission(perm: string): boolean;
    /**
     * Revoke all permissions
     * @memberof VaultItem
     */
    revokeAllPermissions(): void;
    /**
     * Revoke a single permission
     * @param perm The permission to revoke
     * @memberof VaultItem
     */
    revokePermission(perm: string): void;
    /**
     * Clean up all of the data in the vault item
     * @protected
     * @memberof VaultItem
     */
    _cleanUp(): void;
    /**
     * Update source references
     * @protected
     * @memberof VaultItem
     */
    _updateRefs(): void;
}
